function [W mu_opt kernel_opt runtime logliks fval] = learnMapping_tune(X, a, b, choiceModel, params)
    % Learning W directly by call learnMapping_W.
    % Tune mu and RBF kernel parameters jointly by cross validation
    % For tuning mu, the technique similar to continuation method is used
    % the solution with large mu is used for initialization to optimize the
    % smaller one.
    % Output parameters:
    % 
    % W: the weight vector learned
    % mu_opt: the mu selected by cross validation
    % 
    % runtime: the time it takes for learning the mapping, excluding tuning
    % fval: the objective function value
    
    % initial date structure
    numFolds = params.numFolds;
    mus = params.mus;
    numMus = length(mus);
    sigmas = params.kernel.sigmas;
    numSigmas = length(sigmas);
    logliks = zeros(numMus, numSigmas, numFolds);
    
    % split the data into folds then run cross validation to tune mu and
    % sigma jointly
    lin = find((a+b) ~= 0);
    rand('seed',12345);
    folds = mod(randperm(length(lin)), numFolds)+1;
    
    disp('Tuning Parameters\n');
    for f = 1:numFolds
        % set the train/tune datasets
        disp(['Fold ' num2str(f)]);
        a_train = a;
        a_train(lin(folds==f)) = 0;
        a_tune = a;
        a_tune(lin(folds~=f)) = 0;
        
        b_train = b;
        b_train(lin(folds==f)) = 0;
        b_tune = b;
        b_tune(lin(folds~=f)) = 0;
        % try each combination on each folds
        for sIdx = 1 : numSigmas
            params.kernel.sigma = sigmas(sIdx);
            for muIdx =  1:numMus
                params.mu = mus(muIdx);
                disp(['fold = ' num2str(f) ' sigma = ' num2str(params.kernel.sigma) ' mu = ' num2str(params.mu)]);
                W = learnMapping(X, a_train, b_train, choiceModel, params);
                logliks(muIdx, sIdx, f) = computeLoglik(W * Kx(X, X, params.kernel), a_tune, b_tune, choiceModel);
            end
        end
    end
    % select the parameters with highest tuning loglik
    tmp = sum(logliks, 3);
    [tmp, muIdx] = max(tmp);
    [~, sIdx] = max(tmp);
    sigma_opt = sigmas(sIdx);
    params.kernel.sigma = sigma_opt;
    mu_opt = mus(muIdx(sIdx));
    params.mu = mu_opt;
    % run on all training data with optimal parameters selected
    [W runtime fval ] = learnMapping(X, a, b, choiceModel, params);
    kernel_opt = params.kernel;
    