function x = yada_inverse_map(y, W, X, kernel)
    % 
    % x = yada_inverse_map(y, W, X, kernel)
    %
    % Given coordinates in original space, compute the coordinates in
    % transformed space.
    % ***************************************
    % Input Parameters:
    % ***************************************
    % y: d_h-by-n matrix, the coordinates of some objects in the transformed space
    % W: the weight vector learned by yada_learn;
    % X: The coordinates of objects used in input of yada_learn.
    % kernel: kernel parameters used in yada_learn, which is a structure.
    % Currently, only RBF kernel is supported. So kernel.type = 'rbf' and
    % kernel.sigma should be specified. The output of yada_learn.
    % ***************************************
    % Output Parameters:
    % ***************************************
    % x: d_r-by-n matrix, the coordinates of y in raw physical space
    
    X_normalized = (X - repmat(mean(X, 2), 1, size(X, 2))) ./ repmat(std(X, 0, 2), 1, size(X, 2));
    
    n = size(y, 2);
    
    x = zeros(size(X,1), n);
    
    for i=1:n
        x(:,i) = inverseMapping(W, X_normalized, y(:,i), kernel);
    end
    x = x .* repmat(std(X, 0, 2), 1, size(x, 2)) + repmat(mean(X, 2), 1, size(x, 2));